@@ -1,5 +1,5 @@
 <?php
-// $Id: pager_test.php,v 1.15 2005/09/27 07:49:14 quipo Exp $
+// $Id: pager_test.php,v 1.22 2006/06/07 11:48:13 quipo Exp $
 
 require_once 'simple_include.php';
 require_once 'pager_include.php';
@@ -45,10 +45,8 @@ class TestOfPager extends UnitTestCase {
     function testLastPageComplete () {
         $this->assertEqual(true, $this->pager->isLastPageComplete());
     }
-    function testOffsetByPageId1() {
+    function testOffsetByPageId() {
         $this->assertEqual(array(1, 5), $this->pager->getOffsetByPageId(1));
-    }
-    function testOffsetByPageId2() {
         $this->assertEqual(array(6, 10), $this->pager->getOffsetByPageId(2));
     }
     function testOffsetByPageId_outOfRange() {
@@ -56,12 +54,10 @@ class TestOfPager extends UnitTestCase {
     }
     function testGetPageData() {
         $this->assertEqual(array(0=>1, 1=>2, 2=>3, 3=>4, 4=>5), $this->pager->getPageData());
-    }
-    function testGetPageData2() {
         $this->assertEqual(array(5=>6, 6=>7, 7=>8, 8=>9, 9=>10), $this->pager->getPageData(2));
     }
     function testGetPageData_OutOfRange() {
-        $this->assertEqual(false, $this->pager->getPageData(3));
+        $this->assertEqual(array(), $this->pager->getPageData(3));
     }
     function testSelectBox() {
         $selectBox  = '<select name="'.$this->pager->_sessionVar.'">';
@@ -139,7 +135,7 @@ class TestOfPager extends UnitTestCase {
         $options = array(
             'extraVars' => array(
                 'request' => array('aRequest'),
-                'escape'    => '%<>+',
+                'escape'    => 'äö%<>+',
             ),
             'perPage' => 5,
         );
@@ -210,8 +206,12 @@ class TestOfPager extends UnitTestCase {
         */
         $expected = $options['extraVars'];
         $this->assertEqual($expected, $this->pager->_getLinksData());
+        $separator = ini_get('arg_separator.output');
+        if ($separator == '&') {
+            $separator = '&amp;';
+        }
 
-        $expected = '<a href="'.$_SERVER['PHP_SELF'].'?arr%5B0%5D=apple&amp;arr%5B1%5D=orange&amp;no=test&amp;pageID=2" title=""></a>';
+        $expected = '<a href="'.$_SERVER['PHP_SELF'].'?arr%5B0%5D=apple'.$separator.'arr%5B1%5D=orange'.$separator.'no=test'.$separator.'pageID=2" title=""></a>';
         $actual = $this->pager->_renderLink('', '');
         $this->assertEqual($expected, $actual);
     }
@@ -235,8 +235,11 @@ class TestOfPager extends UnitTestCase {
         );
         $actual = $this->pager->_getLinksData();
         $this->assertEqual($expected, $this->pager->_getLinksData());
-
-        $expected = '<a href="'.$_SERVER['PHP_SELF'].'?arr%5B0%5D=apple&amp;arr%5B1%5D=orange&amp;pageID=2" title=""></a>';
+        $separator = ini_get('arg_separator.output');
+        if ($separator == '&') {
+            $separator = '&amp;';
+        }
+        $expected = '<a href="'.$_SERVER['PHP_SELF'].'?arr%5B0%5D=apple'.$separator.'arr%5B1%5D=orange'.$separator.'pageID=2" title=""></a>';
         $actual = $this->pager->_renderLink('', '');
         $this->assertEqual($expected, $actual);
     }
@@ -257,6 +260,19 @@ class TestOfPager extends UnitTestCase {
 
         ini_set('arg_separator.output', $bkp_arg_separator);
     }
+    function testAttributes() {
+        $options = array(
+            'itemData'    => array(1, 2, 3, 4, 5, 6, 7, 8, 9, 10),
+            'perPage'     => 5,
+            'linkClass'   => 'testclass',
+            'attributes'  => 'target="_blank"',
+        );
+        $this->pager =& Pager::factory($options);
+
+        $expected = '<a href="'.$_SERVER['PHP_SELF'].'?pageID=2" class="testclass" target="_blank" title=""></a>';
+        $actual = $this->pager->_renderLink('', '');
+        $this->assertEqual($expected, $actual);
+    }
     function testImportQuery() {
         //add some fake url vars
         $_GET['arr'] = array(
@@ -473,5 +489,72 @@ class TestOfPager extends UnitTestCase {
         $this->pager = Pager::factory($options);
         $this->assertEqual('GET', $this->pager->_httpMethod);
     }
+    function testAccesskey() {
+        $options = array(
+            'itemData' => array(1, 2, 3, 4, 5, 6, 7, 8, 9, 10),
+            'perPage'  => 5,
+            'accesskey' => true,
+        );
+        $this->pager = Pager::factory($options);
+        $this->assertWantedPattern('/accesskey="\d"/i', $this->pager->links);
+        //var_dump($this->pager->links);
+    }
+    function testIsEncoded() {
+    //var_dump(urlencode('&#50504;&#45397;'));
+        $test_strings_encoded = array(
+            'encoded0' => '&#35797;',
+            'encoded1' => '&#27979;&#35797;',
+            'encoded2' => '&#50504;&#45397;',
+            'encoded3' => '&#50504; &#45397;',
+            'encoded4' => '&#50504;
+&#45397;',
+        );
+        $test_strings_plain = array(
+            'plain1' => '안녕',
+            'plain2' => '더보기',
+//          'plain3' => '이젠 전화도
+//로 걸면 무료',
+            'plain4' => 'abcde',    //not multibyte
+            'plain5' => '&#abcfg;', //invalid hex-encoded char
+            'plain5' => '&#50504; nasty &#45397;', //mixed plain/encoded text
+        );
+        foreach ($test_strings_encoded as $string) {
+            //echo '<hr />'.str_replace('&', '&amp;', $string);
+            $this->assertTrue($this->pager->_isEncoded($string));
+        }
+        foreach ($test_strings_plain as $string) {
+            $this->assertFalse($this->pager->_isEncoded($string));
+        }
+    }
+    function testGetOption() {
+        $this->assertEqual(5, $this->pager->getOption('perPage'));
+        $err = $this->pager->getOption('non_existent_option');
+        $this->assertEqual(ERROR_PAGER_INVALID, $err->getCode());
+    }
+    function testGetOptions() {
+        $options = $this->pager->getOptions();
+        $this->assertTrue(is_array($options));
+        $this->assertEqual(5, $options['perPage']);
+    }
+    function testSetOptionsAndBuild() {
+        $options = array(
+            'perPage'  => 2,
+        );
+        $this->pager->setOptions($options);
+        $this->pager->build();
+        $this->assertEqual(2, $this->pager->getOption('perPage'));
+        $this->assertEqual(array(0=>1, 1=>2), $this->pager->getPageData());
+        $this->assertEqual(array(2=>3, 3=>4), $this->pager->getPageData(2));
+
+        $options = array(
+            'currentPage' => 2,
+            'append'   => false,
+            'fileName' => 'myfile.%d.php',
+        );
+        $this->pager->setOptions($options);
+        $this->pager->build();
+        $expected = '<link rel="previous" href="'.$this->baseurl.'/myfile.1.php" title="previous page" />'."\n";
+        $this->assertEqual($expected, $this->pager->_getPrevLinkTag());
+    }
 }
 ?>
\ No newline at end of file
